package com.persagy.person.controller;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.google.common.collect.Lists;
import com.persagy.common.constant.SaasCommonConstant;
import com.persagy.common.enums.ResponseCode;
import com.persagy.common.utils.ResponseResult;
import com.persagy.common.utils.ResponseResultUtil;
import com.persagy.common.utils.StringUtil;
import com.persagy.person.manage.SaasAuthHandler;
import com.persagy.person.manage.SaasRoleMenuHandler;
import com.persagy.person.pojo.dto.SaasMenu;
import com.persagy.person.pojo.vo.menu.SaasMenuCreateVO;
import com.persagy.person.pojo.vo.menu.SaasMenuDeleteVO;
import com.persagy.person.pojo.vo.menu.SaasMenuQueryVO;
import com.persagy.person.service.ISaasMenuService;
import com.persagy.person.service.ISaasRoleMenuService;

import cn.hutool.core.collection.CollectionUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

/**
 * 菜单信息
 * 
 * @version 1.0.0
 * @company persagy 
 * @author zhangqiankun
 * @date 2021-03-13 15:29:50
 */
@Api(tags = "菜单信息")
@RestController
@RequestMapping(value = "/menu", method = RequestMethod.POST)
public class SaasMenuController {
	
	@Autowired
	private SaasAuthHandler saasAuthHandler;
	
    @Autowired
    private ISaasMenuService saasMenuService;
    
    @Autowired
    private SaasRoleMenuHandler saasRoleMenuHandler;
    
    @Autowired
    private ISaasRoleMenuService saasRoleMenuService;
    
    /**
     * 查询账号菜单功能权限树，根据账号ID做权限过滤，主要供账号登录后左侧展示使用
     */
    @ApiOperation(value = "查询账号菜单功能权限树")
    @RequestMapping(value = "querySaasMenuTree")
    public ResponseResult querySaasMenuTree(@RequestBody @Validated SaasMenuQueryVO queryVO) {
    	// 获取此账号对应的菜单范围
    	Map<String, String> menuAuth = this.saasAuthHandler.validSaasMenuAuth(queryVO.getAccountId(), queryVO.getGroupCode());
    	if (StringUtil.isNotBlank(menuAuth.get(SaasCommonConstant.GROUP_CODE))) {
    		queryVO.setGroupCode(menuAuth.get(SaasCommonConstant.GROUP_CODE));
		}
    	// 添加数据访问控制，账号可见域，0-所有集团项目，1-单集团所有项目(此时，集团编码需存在)，2-其他
    	String accountType = menuAuth.get(SaasCommonConstant.ACCOUNT_TYPE);
    	// 账号所属，0-运维系统，1-业务系统账号
    	String accountBelong = menuAuth.get(SaasCommonConstant.ACCOUNT_BELONG);
    	List<SaasMenu> menuTree = null;
    	if (SaasCommonConstant.STR_STATUS_0.equals(accountType) && SaasCommonConstant.STR_STATUS_0.equals(accountBelong)) {
    		// 如果是运维系统的账号，accountType为0的情况下查询所有菜单权限，其余情况走表关联查询，为1时，后续可添加
    		menuTree = this.saasRoleMenuHandler.querySaasMenuTree(null, null, true, queryVO.isMenu());
    	} else {
    		menuTree = this.saasRoleMenuHandler.querySaasMenuTree(queryVO.getAccountId(), queryVO.getGroupCode(), false, queryVO.isMenu());
		}
    	return ResponseResultUtil.successResult(menuTree);
    }
    
    /**
     * 查询菜单功能树，角色配置使用
     */
    @ApiOperation(value = "查询菜单功能树")
    @RequestMapping(value = "queryMenuFunTree")
    public ResponseResult queryMenuFunTree(@RequestBody @Validated SaasMenuQueryVO queryVO) {
    	if (StringUtil.isBlank(queryVO.getMenuType())) {
			return ResponseResultUtil.errorResult(ResponseCode.A0400.getCode(), "菜单类型不可为空");
		}
    	// 这里按照超管的逻辑进行处理，因为不会做权限过滤
		LambdaQueryWrapper<SaasMenu> queryWrapper = new SaasMenu.Builder().createQueryWrapper().idEq(queryVO.getMenuId())
				.menuTypeEq(queryVO.getMenuType()).appIdEq(queryVO.getAppId()).builderQueryWrapper();
		List<SaasMenu> menuTree = this.saasMenuService.list(queryWrapper);
		if (CollectionUtil.isEmpty(menuTree)) {
			return ResponseResultUtil.successResult(Lists.newArrayList());
		}
		// 2.转为菜单树
		List<SaasMenu> topMenus = new ArrayList<SaasMenu>();
		for (int i = menuTree.size() - 1; i >= 0; i--) {
			SaasMenu saasMenu = menuTree.get(i);
			if (StringUtil.isBlank(saasMenu.getParentId())) {
				topMenus.add(saasMenu);
				menuTree.remove(i);
			}
		}
		List<SaasMenu> parents = topMenus.stream().sorted(Comparator.comparing(SaasMenu::getMenuSort)).collect(Collectors.toList());
		this.saasRoleMenuHandler.setMenuChildrens(null, menuTree, parents, true, true);
    	return ResponseResultUtil.successResult(parents);
    }
    
    /**
     * 新增
     */
	@ApiOperation(value = "保存")
	@RequestMapping(value = "createSaasMenu")
    public ResponseResult createSaasMenu(@RequestBody @Validated SaasMenuCreateVO createVO) {
		if (StringUtil.isNotBlank(createVO.getMenuUrl()) && StringUtil.isEmpty(createVO.getUrlType())) {
			return ResponseResultUtil.errorResult(ResponseCode.A0400.getCode(), "当菜单URL存在时，URL类型不可为空");
		}
		SaasMenu saasMenu = new SaasMenu();
		BeanUtils.copyProperties(createVO, saasMenu);
		saasMenu.setId(null);
		saasMenu.setUpdateUser(createVO.getAccountId());
        boolean result = this.saasMenuService.save(saasMenu);
        return result ? ResponseResultUtil.successResult("保存成功", saasMenu.getId()) : ResponseResultUtil.errorResult("保存失败");
    }
    
    /**
     * 更新
     */
	@ApiOperation(value = "更新")
	@RequestMapping(value = "updateSaasMenu")
    public ResponseResult updateSaasMenu(@RequestBody @Validated SaasMenuCreateVO createVO) {
		if (StringUtil.isBlank(createVO.getId())) {
			return ResponseResultUtil.errorResult(ResponseCode.A0400.getCode(), "菜单ID不可为空");
		}
		if (StringUtil.isNotBlank(createVO.getMenuUrl()) && StringUtil.isEmpty(createVO.getUrlType())) {
			return ResponseResultUtil.errorResult(ResponseCode.A0400.getCode(), "当菜单URL存在时，URL类型不可为空");
		}
		SaasMenu saasMenu = new SaasMenu();
		BeanUtils.copyProperties(createVO, saasMenu);
		saasMenu.setUpdateUser(createVO.getAccountId());
        boolean result = saasMenuService.updateById(saasMenu);
        return result ? ResponseResultUtil.successResult("更新成功") : ResponseResultUtil.errorResult("更新失败");
    }

    /**
     * 物理删除
     */
    @ApiOperation(value = "删除")
    @RequestMapping(value = "deleteSaasMenu")
    public ResponseResult deleteSaasMenu(@RequestBody @Validated SaasMenuDeleteVO deleteVO) {
    	// 只有此菜单不属于任何可用的角色才允许被删除
    	boolean result = this.saasRoleMenuService.isAllowDeleteMenu(deleteVO.getGroupCode(), deleteVO.getId());
    	if (!result) {
    		return ResponseResultUtil.errorResult(ResponseCode.A0402.getCode(), "该菜单已被使用，不允许被删除");
		}
    	result = this.saasRoleMenuHandler.deleteSaasMenu(deleteVO.getId());
        return result ? ResponseResultUtil.successResult("删除成功") : ResponseResultUtil.errorResult("删除失败");
    }
    
}
